
@=========================================================================@
/* A DYNAMIC FACTOR MODEL :  BASED ON STOCK AND WATSON (1989)            */

/*        MODEL: (based on demeaned log differences)

          yt = gamma_i*(Delta c_t) + z_{it},  i=1,2,3; t=1,...,T
      y_t =gamma_{40}(Delta c_t)+gamma_{41}(Delta c_{t-1})+
           +gamma_{42}(Delta c_{t-2})+gamma_{43}(Delta c_{t-3})+z_{4t},
         (Delta c_t)= phi1* (Delta c_{t-1})+(Delta c_{t-2}) + e_t
          z_{it}=psi_{i1}*z_{i,t-1} +psi_{i2}*z_{i,t-2} + v_{it}
          e_t -- iid N(0,1)
          v_{it}--iid N(0,sig_i^2)


           PARAMETERS TO BE ESTIMATED:
            phi1,phi2,
            psi_{i1},psi_{i2}, sig_i^2, i=1,2,3,4,
        gamma_i, i=1,2,3; gamma_{40}, gamma_{41}, gamma_{42}, gamma_{43}


WRITTEN BY CHANG-JIN KIM, DEPT. OF ECONOMICS, KOREA UNIVERSITY
           (cjkim@.korea.ac.kr)
Based on Stock and Watson (1991), ``A Probability Model of the Coincident
Economic Indicators,''  in K. Lahiri and G.H. Moore (eds.),
{\it Leading Economic Indicators: New Approaches and Forecasting Records,}
Cambridge: Cambridge University Press.

                                                                           */
@===========================================================================@

new;
library optmum, pgraph_gnu;

format /m1 /rd 8,4;


load data[433,6]=s&w.prn;  @1959.1 -- 1995.1@
                @ month, ip, gmyxpq, mtq, lpnag, DOC index@


lndata=ln(data);

              xx=seqa(1,1,rows(data));
              begwind;
          window(2,2,0);
                setwind(1);
            title("y1");
        xy(xx,data[.,2]);
                nextwind;
            title("y2");
        xy(xx,data[.,3]);
                nextwind;
            title("y3");
        xy(xx,data[.,4]);
                nextwind;
            title("y4");
        xy(xx,data[.,5]);
        endwind;

month=data[2:433,1];

y1=(ln(data[2:433,2]) - ln(data[1:432,2]))*100;
y2=(ln(data[2:433,3]) - ln(data[1:432,3]))*100;
y3=(ln(data[2:433,4]) - ln(data[1:432,4]))*100;
y4=(ln(data[2:433,5]) - ln(data[1:432,5]))*100;

doc_in= (data[2:433,6]);
doc_din= ((data[2:433,6])-(data[1:432,6]));
doc_mn=meanc(doc_din);
doc_sd=stdc(doc_din);


yy0=(y1~y2~y3~y4);


/*
@================================Option 1: Based on====================
                             Standardized and Demeaned data
                 [Easy to get convergence!!]
=======================================================================@
yy_s=stdc(yy0);
yy_org= (yy0'./yy_s);
yy_m=meanc(yy_org');
yy=(yy_org - yy_m);  @Standardized y1, y2,y3,y4:  @
                          @NOTE:  DIMENSION= 4xT  @

prmtr_in={0.5  0.1  0.5 0.1 0.5 0.1 0.5 0.1 0.5  0.1
          0.5 0.5  0.5 0.5
          0.5 0.5 0.5 0.5 0 0 0                };
@=======================================================================@
*/

@================================Option 2: Based on======================
                     Demeaned data (without standardizing)
             [Difficult to get convergence if starting values are
             not good enough!!!]
========================================================================@

yy_m=meanc(yy0);
yy=yy0'-yy_m;   @4xT@

prmtr_in={0.2  0.1  0.2 0.1 0.2 0.1 0.2 0.1 0.2  0.1
          0.5 0.6  0.8 0.15
          0.5 0.2 0.5 0.1 0 0 0                };

prmtr_in={
  1.2961  -0.142  -0.0473  -0.0473  -0.1874  -0.1874  -0.2272  -0.2272
  1.3428  -1.4954   0.4939   0.5549   0.8008   0.1382   0.6412   0.2420
  0.5071   0.1402   0.0810   0.1055   0.4382 };

@=====================================================================@


t=cols(yy);


@========Maximum Likelihood Estimation=========================================@


prmtr_in=prmtr_in';


/*

{xout,fout,gout,cout}=optmum(&lik_fcn,prmtr_in);

prm_fnl=trans(xout);

       "Calculating Hessian, and inverting..... Please be patient!!!!";
                  hessn0=hessp(&lik_fcn,xout);
                  cov0=inv(hessn0);

grdn_fnl=gradfd(&trans,xout);
cov=grdn_fnl*cov0*grdn_fnl';
sd=sqrt(diag(cov));                 @...Standard errors of the estimated coeff.@


output file=s&w.out reset;

"--------------------------------------------------";
"log likelihood value is ";; -fout;
"-------------------------------------------------- ";

"Estimated parameters and standard errors are:";
" ";
prm_fnl~sd;
" ";
"---------------------------------------------------";
"Xout =";
xout';
" ";
"For Normal Convergence, The following number is 0:";
cout;
"---------------------------------------------------";
output off;
*/

/* Don't forget to delete the following line when you estimate the
parameters of the model */

xout=prmtr_in;


{ft_ct,delta_}=filter(xout);

f_err=ft_ct[.,1:4];

dlt_ctt=ft_ct[.,5];
sd_ctt=stdc(dlt_ctt);  @standard dev. of (Delta c_{t|t})@

        new_ind=zeros(t,1);

       ii=2;
        do until ii>t;
              new_ind[ii,1]=dlt_ctt[ii,1] + new_ind[ii-1,1] + delta_;
        ii=ii+1;
        endo;

new_cci=
  (new_ind)*doc_sd/sd_ctt;

new_cci=new_cci+doc_in[120,1]-new_cci[120,1];

output file=s&w.dta reset;
month~new_cci~doc_in;
output off;

output file=garbage.prn reset; lndata[2:rows(data),2:5]~new_cci~doc_in;
output off;

title("DOC index vs. Experimental Index");
xy(seqa(1,1,rows(new_ind)),new_cci~doc_in);



@========End of Main Program===================================================@

proc lik_fcn(prmtr1);

  local sig_e,sig,F,H,Q,R,BETA_LL,P_LL,BETA_TL,P_TL,BETA_TT,P_TT,vt,ft,
        valu,loglik,j_iter,vecP_LL,prmtr,phi1,phi2,psi11,psi12,psi21,
        psi22,psi31,psi32,psi41,psi42,sig1,sig2,sig3,sig4,val,gamma1,
        gamma2,gamma3,gamma4,gamma41,gamma42,gamma43;

    loglik=zeros(t,1);

    prmtr=trans(prmtr1);

locate 20,1; prmtr';

    phi1=prmtr[1,1];
    phi2=prmtr[2,1];
    psi11=prmtr[3,1];
    psi12=prmtr[4,1];
    psi21=prmtr[5,1];
    psi22=prmtr[6,1];
    psi31=prmtr[7,1];
    psi32=prmtr[8,1];
    psi41=prmtr[9,1];
    psi42=prmtr[10,1];

    sig1=prmtr[11,1];
    sig2=prmtr[12,1];
    sig3=prmtr[13,1];
    sig4=prmtr[14,1];

    gamma1=prmtr[15,1];
    gamma2=prmtr[16,1];
    gamma3=prmtr[17,1];
    gamma4=prmtr[18,1];

    gamma41=prmtr[19,1];
    gamma42=prmtr[20,1];
    gamma43=prmtr[21,1];

@=====================================================================@

 F=(phi1~phi2~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   1~   0~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   1~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  1~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~psi11~psi12~  0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    1~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~psi21~psi22~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   1~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~psi31~psi32~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  1~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  0~  0~psi41~psi42)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  0~  0~  1~  0);

   H=(gamma1~0~0~0~                  1~0~0~0~0~0~0~0)|
     (gamma2~0~0~0~                  0~0~1~0~0~0~0~0)|
     (gamma3~0~0~0~                  0~0~0~0~1~0~0~0)|
     (gamma4~gamma41~gamma42~gamma43~0~0~0~0~0~0~1~0);

    sig_e=1;

    Q=(sig_e~0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~  sig1~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~sig2~0~   0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~sig3~0~   0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~sig4~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0);

    R=0;

@================================================================@

    BETA_LL=0|0|0|0|0|0|0|0|0|0|0|0;

    vecP_LL=inv(eye(12^2) - F.*.F)*vec(Q);              @..Vectorization..@

    P_LL=vecP_LL[1:12,1]~vecP_LL[13:24,1]~vecP_LL[25:36,1]~vecP_LL[37:48,1]
        ~vecP_LL[49:60,1]~vecP_LL[61:72,1]~vecP_LL[73:84,1]~vecP_LL[85:96,1]
        ~vecP_LL[97:108,1]~vecP_LL[109:120,1]~vecP_LL[121:132,1]~
         vecP_LL[133:144,1];

                           @..initial state vector and its covariance ..@

    j_iter=1;
    do until j_iter > t;

        BETA_TL = F*BETA_LL;
        P_TL = F*P_LL*F' + Q;

        vt = yy[.,j_iter] - H*BETA_TL;
        ft = H*P_TL*H' + R;

        BETA_TT = BETA_TL + P_TL*H'*inv(ft)*vt;
        P_TT = P_TL - P_TL*H'*inv(ft)*H*P_TL;


        BETA_LL=BETA_TT;
        P_LL=P_TT;


            val=v_prob(vt,ft);                @likelihood@
            loglik[j_iter,1] = -1*ln(val);    @ - log likelihood@


    j_iter=j_iter+1;
    endo;

retp(sumc(loglik));
endp;

@==============================================================================@

@==============================================================================@

proc (2)= filter(prmtr1);

  local sig_e,sig,F,H,Q,R,BETA_LL,P_LL,BETA_TL,P_TL,BETA_TT,P_TT,vt,ft,
        valu,loglik,j_iter,vecP_LL,prmtr,phi1,phi2,psi11,psi12,psi21,
        psi22,psi31,psi32,psi41,psi42,sig1,sig2,sig3,sig4,val,gamma1,
        gamma2,gamma3,gamma4,out_mat,c_bar,ss_kg,k,alpfa,
        gamma41,gamma42,gamma43;

    prmtr=trans(prmtr1);



    phi1=prmtr[1,1];
    phi2=prmtr[2,1];
    psi11=prmtr[3,1];
    psi12=prmtr[4,1];
    psi21=prmtr[5,1];
    psi22=prmtr[6,1];
    psi31=prmtr[7,1];
    psi32=prmtr[8,1];
    psi41=prmtr[9,1];
    psi42=prmtr[10,1];

    sig1=prmtr[11,1];
    sig2=prmtr[12,1];
    sig3=prmtr[13,1];
    sig4=prmtr[14,1];

    gamma1=prmtr[15,1];
    gamma2=prmtr[16,1];
    gamma3=prmtr[17,1];
    gamma4=prmtr[18,1];

    gamma41=prmtr[19,1];
    gamma42=prmtr[20,1];
    gamma43=prmtr[21,1];

@=====================================================================@

 F=(phi1~phi2~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   1~   0~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   1~  0~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  1~  0~    0~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~psi11~psi12~  0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    1~   0~   0~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~psi21~psi22~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   1~   0~  0~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~psi31~psi32~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  1~  0~  0~  0)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  0~  0~psi41~psi42)|
   (   0~   0~  0~  0~    0~   0~   0~   0~  0~  0~  1~  0);

   H=(gamma1~0~0~0~                  1~0~0~0~0~0~0~0)|
     (gamma2~0~0~0~                  0~0~1~0~0~0~0~0)|
     (gamma3~0~0~0~                  0~0~0~0~1~0~0~0)|
     (gamma4~gamma41~gamma42~gamma43~0~0~0~0~0~0~1~0);

    sig_e=1;

    Q=(sig_e~0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~   0~0~0~    0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~  sig1~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~sig2~0~   0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~sig3~0~   0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~sig4~0)|
      (   0~0~0~0~       0~0~    0~0~    0~0~    0~0);

    R=0;

@================================================================@

    BETA_LL=0|0|0|0|0|0|0|0|0|0|0|0;

    vecP_LL=inv(eye(12^2) - F.*.F)*vec(Q);              @..Vectorization..@

    P_LL=vecP_LL[1:12,1]~vecP_LL[13:24,1]~vecP_LL[25:36,1]~vecP_LL[37:48,1]
        ~vecP_LL[49:60,1]~vecP_LL[61:72,1]~vecP_LL[73:84,1]~vecP_LL[85:96,1]
        ~vecP_LL[97:108,1]~vecP_LL[109:120,1]~vecP_LL[121:132,1]~
         vecP_LL[133:144,1];


    out_mat=zeros(t,5);


    j_iter=1;
    do until j_iter > t;

        BETA_TL = F*BETA_LL;

        P_TL = F*P_LL*F' + Q;

        vt = yy[.,j_iter] - H*BETA_TL;
        ft = H*P_TL*H' + R;

        BETA_TT = BETA_TL + P_TL*H'*inv(ft)*vt;
        P_TT = P_TL - P_TL*H'*inv(ft)*H*P_TL;


        BETA_LL=BETA_TT;
        P_LL=P_TT;

        out_mat[j_iter,.]=vt'~beta_tt[1,1];


    j_iter=j_iter+1;
    endo;

    ss_kg=P_TL*H'*inv(ft);              @..Steady-State Kalman Gain..@
    k=(eye(12)-ss_kg*h)*f;
    alpfa=inv(eye(12) - k)*ss_kg*yy_m;
    c_bar=alpfa[1,1];



retp(out_mat, c_bar); @Returns out_mat and the mean of the common component@
endp;




@==============================================================================@

proc v_prob(err,var);                          @..Calculates  Pr[Yt/St, Yt-1]..@

  local valu;

    valu = (1/sqrt(det(var)))*exp(-0.5*err'inv(var)*err);

retp(valu);

endp;

@==============================================================================@

proc trans(c0);                           @..Constraining Values of Reg. Coff..@

  local c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,cc;

    cc=c0;

    c1=c0[1,1]/(1+abs(c0[1,1]));
    c2=c0[2,1]/(1+abs(c0[2,1]));
    cc[1,1]=c1+c2;
    cc[2,1]=-1*c1*c2;

    c3=c0[3,1]/(1+abs(c0[3,1]));
    c4=c0[4,1]/(1+abs(c0[4,1]));
    cc[3,1]=c3+c4;
    cc[4,1]=-1*c3*c4;

    c5=c0[5,1]/(1+abs(c0[5,1]));
    c6=c0[6,1]/(1+abs(c0[6,1]));
    cc[5,1]=c5+c6;
    cc[6,1]=-1*c5*c6;

    c7=c0[7,1]/(1+abs(c0[7,1]));
    c8=c0[8,1]/(1+abs(c0[8,1]));
    cc[7,1]=c7+c8;
    cc[8,1]=-1*c7*c8;

    c9=c0[9,1]/(1+abs(c0[9,1]));
    c10=c0[10,1]/(1+abs(c0[10,1]));
    cc[9,1]=c9+c10;
    cc[10,1]=-1*c9*c10;

    cc[11:14]=c0[11:14]^2;

    cc[19:21]=c0[19:21]/10;


retp(cc);

endp;


