@>>>>>>>>>>>>>>>>>>>>Time varying parameter model  >>>>>>>>>>>@
@>>>>>>>>>>>>>>>>>>>>Using Optmum Proc  >>>>>>>>>>>>>>>>>>>>>>@

/*  Program used in Kim and Nelson (1989, Journal of Business and
    Economic Statistics), "Time-Varying-Parameter Model for Modeling
    Changing Conditional Variance: The Case of the Lucas Hypothesis" 

cjkim@korea.ac.kr
 */

New;
library optmum,pgraph;

/* Load Data  */

load data[106,6]=TVP.prn;

     /* column 1: Quarter Index
               2: m1===growth rate of quarterly average M1
               3: dint=change in the lagged interest rate (3-month T-bill)
               4: inf==lagged inflation
               5: surpl==lagged full employment budget surplus
               6: m1lag==lag of m1
                 1959.3--1987.4,  ........................ */
qtr_idx=data[1:106,1];
data=data[1:106,2:6];
T=rows(data);

yy=data[.,1];XX=ones(T,1)~data[.,2:5];


@ m1, intcpt, dint, inf,  surp,m1lag@




@================= Initialize Global Variables============@


         START=11;    @1961.1--1987.4 @
         PRMTR_NO=6;   @ >>Number of parameters to be estimated. >>@
         TVP_NO=5;     @ >>Number of Time-varying Coefficients   >>@

           pr_1=0.5;pr_2=0.1;pr_3=0.1;
           pr_4=0.1;pr_5=0.1;pr_6=0.1;
            @initial values of parameters@

         PRMTR_IN=pr_1|pr_2|pr_3|pr_4|pr_5|pr_6;




@ Maximum Likelihood Estimation @


 {xout,fout,gout,cout}=optmum(&lik_fcn,PRMTR_IN);

       @ prmtr estimates, -log lik value, Grandient, Hessian@

        "Calculating Hessian..... Please be patient!!!!";

                  COV=inv(hessp(&lik_fcn,xout));


PRM_FNL=xout;             @ Estimated  coefficients, constrained@

SD_fnl =sqrt(diag(COV)); @Standard errors of the estimated coefficients@


format /m1 /rd 8,4;
output file=tvp.out reset;

"==FINAL OUTPUT========================================================";
"likelihood value is ";; -fout;
"Estimated parameters are:";
prm_fnl';
XOUT';
"Standard errors of parameters are:"; sd_fnl';
"===============================================================";
output off;


{F_SS,tvp_coef}=SAVE_OUT(PRM_FNL);

OUTPUT FILE=TVP.DTA RESET;
F_SS~tvp_coef; OUTPUT OFF;

begwind;
window(2,2,0);
xlabel("quarter");
xindex=seqa(1,1,rows(f_ss));

setwind(1);
   title("forecast error");
   xy(xindex, f_ss[.,2]);

nextwind;
   title("conditional variance");
   xy(xindex, f_ss[.,3]);

nextwind;
   title("beta0t");
   xy(xindex,tvp_coef[.,1]);

nextwind;
   title("beta1t");
   xy(xindex,tvp_coef[.,2]);

endwind;

begwind;
window(2,2,0);
setwind(1);
   title("beta2t");
   xy(xindex, tvp_coef[.,3]);

nextwind;
   title("beta3t");
   xy(xindex, tvp_coef[.,4]);

nextwind;
   title("beta4t");
   xy(xindex,tvp_coef[.,5]);

endwind;

@========================================================================@
@========================================================================@
PROC LIK_FCN(PRMTR);

local BETA_TT, P_TT, QQ, LIK, J_ITER, XT, BETA_TL, P_TL, SS,K_GAIN,
      F_CAST, F, BETA_LL, P_LL,SIG_E,SIG_1,SIG_2,SIG_3,SIG_4,SIG_5,RR,
      XT,YT;



     SIG_E=PRMTR[1,1];
     SIG_1=PRMTR[2,1];
     SIG_2=PRMTR[3,1];
     SIG_3=PRMTR[4,1];
     SIG_4=PRMTR[5,1];
     SIG_5=PRMTR[6,1];


     F=EYE(TVP_NO);
     RR=SIG_E^2;
     QQ=(SIG_1^2~0~0~0~0)|(0~SIG_2^2~0~0~0)|(0~0~SIG_3^2~0~0)|
        (0~0~0~SIG_4^2~0)|(0~0~0~0~SIG_5^2);


     BETA_LL=zeros(5,1);      @initial guess for coefficients@
     P_LL=EYE(TVP_NO)*50;     @uncertainty associated with initial guess@


LIK=0.0;
J_ITER = 1;
DO UNTIL J_ITER>T;

      XT=XX[J_ITER,.]; YT=YY[J_ITER,.];

      BETA_TL = F * BETA_LL;
      P_TL = F * P_LL * F' +QQ;

      F_CAST = YT - XT * BETA_TL;
      SS = XT * P_TL * XT' + RR;

      BETA_TT = BETA_TL + (P_TL*XT'/SS) * F_CAST;
      P_TT = (EYE(TVP_NO) - (P_TL*XT'/SS)* XT ) * P_TL;

      BETA_LL=BETA_TT;  P_LL=P_TT;

      IF J_ITER < START; goto skip; endif;

      LIK = LIK+  0.5*(LN(2*pi*SS) + (F_CAST^2)/SS);

skip:  J_ITER = J_ITER+1;
ENDO;

RETP(LIK);
ENDP;

@========================================================================@

PROC (2) = SAVE_OUT(PRMTR);

local BETA_TT, P_TT, QQ, LIK, J_ITER, XT, BETA_TL, P_TL, SS,K_GAIN,
      F_CAST, PRMTR,F,DTA_MAT,dta_mat2,BETA_LL, P_LL,SIG_E,SIG_1,
      SIG_2,SIG_3,SIG_4,SIG_5,RR,YT;

   DTA_MAT=zeros(t,2);
   DTA_MAT2=zeros(t,5);


     SIG_E=PRMTR[1,1];
     SIG_1=PRMTR[2,1];
     SIG_2=PRMTR[3,1];
     SIG_3=PRMTR[4,1];
     SIG_4=PRMTR[5,1];
     SIG_5=PRMTR[6,1];


     F=EYE(TVP_NO);

     RR=SIG_E^2;
     QQ=(SIG_1^2~0~0~0~0)|(0~SIG_2^2~0~0~0)|(0~0~SIG_3^2~0~0)|
        (0~0~0~SIG_4^2~0)|(0~0~0~0~SIG_5^2);


     BETA_LL=zeros(5,1);      @initial guess for coefficients@
     P_LL=EYE(TVP_NO)*50;     @uncertainty associated with initial guess@




   J_ITER = 1;
   DO UNTIL J_ITER>T;


       XT=XX[J_ITER,.]; YT=YY[J_ITER,.];


       BETA_TL = F * BETA_LL;
       P_TL =    F * P_LL * F' +QQ;

       F_CAST = YT - XT * BETA_TL;
       SS = XT * P_TL * XT' + RR;

       BETA_TT = BETA_TL + (P_TL*XT'/SS) * F_CAST;
       P_TT = (EYE(TVP_NO) - (P_TL*XT'/SS) * XT ) * P_TL;



       DTA_MAT[J_ITER,.]=F_CAST~SS;  @Forecast error and conditional variance@
       DTA_MAT2[J_ITER,.]=BETA_TL';


       BETA_LL=BETA_TT;  P_LL=P_TT;


  J_ITER = J_ITER+1;
  ENDO;


RETP(qtr_idx[START:t,.]~dta_mat[START:t,.],dta_mat2[START:t,.]);
ENDP;

END;

