/*
========================================================================
MODEL:  HAMILTON'S(1989,ECONOMETRICA) MARKOV-SWITCHING MODEL


   ***** THIS PROGRAM CONTAINS KIM'S(1994) SMOOTHING ALGORITHM*****


         (y_t-mu_{s_t})= phi_1*(y_{t-1}-mu_{s_{t-1}})+...
                        +phi_4*(y_{t-4}-mu_{s_{t-4}}) + e_t

                 e_t -- i.i.d. N(0,sigma^2)
         mu_{st} = (1-S_t)*mu_0 + S_t* mu_1
         Pr[S_t=1|S_{t-1}=1]=p
                 Pr[S_t=0|S_{t-1}=0]=q

WRITTEN BY CHANG-JIN KIM,        based on Hamilton (1989)
           DEPT. OF ECONOMICS
           KOREA UNIVERSITY
           SEOUL, 136-701, KOREA
       cjkim@korea.ac.kr
=========================================================================
*/

new;
library optmum,PGRAPH;

format /m1 /rd 9,5;
load data[195,1]=gdp4795.prn;  @Quarter, U.S. real GNP@
                              @1947.1 -- 1995.3 @




yy_d=(ln(data[21:152,1])-ln(data[20:151,1]))*100;
                              @1952.1 -- 1984.4 [hamilton data set]@
t0=rows(yy_d);


LAG_AR=4;
NO_ST=lag_ar+1; @ NUMBER OF STATES TO BE CONSIDERED@
DMNSION=2^NO_ST;



st_mat=zeros(DMNSION,NO_ST);

       j=1;
       st4=0;do until st4>1;
       st3=0; do until st3>1;
       st2=0; do until st2>1;
       st1=0; do until st1>1;
       st=0; do until st>1;

      st_mat[j,.]=st4~st3~st2~st1~st;

    j=j+1;
    st=st+1;endo;
    st1=st1+1;endo;
    st2=st2+1;endo;
    st3=st3+1;endo;
    st4=st4+1;endo;

yy=yy_d[lag_ar+1:t0,1];

x_mat=
   yy_d[Lag_ar:T0-1,1]  ~ yy_d[lag_ar-1:T0-2,1]~
   yy_d[lag_ar-2:T0-3,1]~ yy_d[lag_ar-3:T0-4,1];

T=rows(yy);




@================= Initialize Global Variables======1959.1-89.9======@

           START=1;  @1952:4.....@

PRMTR_IN={
  2.20657   1.15590   0.08983  -0.01861  -0.17434  -0.08392   0.79619
 -0.21320   1.12828  };
PRMTR_IN=PRMTR_IN';

@ Maximum Likelihood Estimation @
@==================================================@

{xout,fout,gout,cout}=optmum(&lik_fcn,PRMTR_in);

PRM_FNL=TRANS(xout);             @ Estimated  coefficients, constrained@


output file=hmt4_kim.out reset;     OUTPUT OFF;


"==FINAL OUTPUT========================================================";

"initial values of prmtr is";
                  trans(prmtr_in)';
"==============================================================";
"code is--------";;cout;
"likelihood value is ";; -1*fout;
"Estimated parameters are:";
prm_fnl'; " ";
xout';
output off;


"Calculating Hessian..... Please be patient!!!!";
                  hout0=hessp(&lik_fcn,xout);
                  hout=inv(hout0);

grdn_fnl=gradfd(&TRANS,xout);
Hsn_fnl=grdn_fnl*hout*grdn_fnl';
SD_fnl =sqrt(diag(Hsn_fnl)); @Standard errors of the estimated coefficients@

output on;
"Standard errors of parameters are:"; sd_fnl';
"===============================================================";
output off;


/* Don't forget to delete the following line when you estimate the
parameters.... 

xout=prmtr_in; prm_fnl=trans(xout);
*/



{pr_tt0,pr_tl0}=FILTER(XOUT);  @Pr[S_t=0|Y_t] and Pr[S_t=0|Y_{t-1}]@

smooth0=SMOOTH(pr_tt0,pr_tl0); @Pr[S_t=0|Y_T], Smoothed probabilities@


FLTR=pr_tt0~pr_tl0~smooth0;
format /m1 /rd 8,4;
output file=hmt4_kim.dta reset; FLTR; output off;


xy(seqa(1,1,rows(pr_tt0)),pr_tt0);
xy(seqa(1,1,rows(smooth0)),smooth0);


end;

@ END OF MAIN PROGRAM @




@========================================================================@
@========================================================================@
PROC LIK_FCN(PRMTR1);
local prmtr, ppr,qpr,prob__0,prob__1,QQ, lik, j_iter,  pr__0_l,pr__1_l,
      F_cast, var_L,pr_vl, pr_val,likv,phi,PSIC,PSIX,
      vecp,st,st1,st2,st3,st4,ST5,ST6,ST7,ST8,ST9,ST10,ST11,ST12,ST13,
      pr_tr,pr_trf1,pr_trf,prob__t,prob__,pro_,j,psi1,psi2,var_c,
      delta0,DELTA1,MU0,MU1,st_k,st_k1,st_k2,st_k3,st_k4,
      f_cast1,f_cast2,PR_VL1,PR_VL2,pr_trf7,pr_trf0,
      PR_TRF2,PR_TRF3,PR_TRF4,PR_TRF5,PR_TRF6,psiL,
      TMPRY1,TMPRY2,SM_PRL,TMP_P0,SM_PR0,JJJ,MU_MAT,D_MAT,FLT_PRN,
      F1,F2,TMP00,TMP0,SM_PR00,prob_dd,VAR,A,EN;

PRMTR=TRANS(PRMTR1);

LOCATE 15,1; PRMTR';

/* DEFINE PARAMETERS */

           PPR=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI=PRMTR[3:6,1];
           VAR=PRMTR[7,1]^2;
           MU0=PRMTR[8,1];  @ recession, vs. boom@
           MU1=PRMTR[9,1];  @ recession, vs. boom@


           MU_MAT=ST_MAT*MU1 + (ONES(DMNSION,NO_ST)-ST_MAT)*MU0;

/* A MATRIX OF TRANSITION PROBABILITIES */

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

/* INITIALIZING THE FILTER WITH STEADY-STATE PROBABILITIES */

           A = (eye(2)-pr_tr)|ones(1,2);
           EN=(0|0|1);
           PROB__T = INV(A'A)*A'EN;  @PR[S_t=0]|PR[S_t=1],
                                  2x1 steady-state PROBABILITIES@

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;
           PR_TRF =PR_TRF2|PR_TRF2;


           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF0;
                                 /*PR[S_{-2},S_{-3}|I_0)    4x1*/
           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF1;
                                 /*PR[S_{-1},S_{-2},S_{-3}|I_0)  8x1*/
           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF2;
                                 /*PR[S_{0},S_{-1},S_{-2},S_{-3}|I_0) 16x1 */
           PROB__=VECR(PROB__T~PROB__T);
                                 /*2^5 x 1*/



  LIKV=0.0;
  J_ITER=1;
  DO UNTIL J_ITER>T;


   F_CAST1=(YY[J_ITER,1]-X_MAT[J_ITER,.]*PHI)*ONES(DMNSION,1)
               -(MU_MAT[.,5]  - MU_MAT[.,4]*PHI[1,1]
                              - MU_MAT[.,3]*PHI[2,1]
                              - MU_MAT[.,2]*PHI[3,1]
                              - MU_MAT[.,1]*PHI[4,1] );          /* 2^5x1 */

   VAR_L=VAR*ONES(DMNSION,1);                                    /* 2^5x1 */

   PROB_DD=PR_TRF .* PROB__;
                    /* Pr[S_t,S_{t-1},S_{t-2},S_{t-3},S_{t-4} | I(t-1)]*/

   PR_VL=(1./SQRT(2.*PI.*VAR_L)).*EXP(-0.5*F_CAST1.*F_CAST1./VAR_L).*PROB_DD;
                                                                 /* 2^5x1 */
                               /* Joint density of y_t,S_t,..,S_{t-4} given
                                  past information : */

   PR_VAL=SUMC(PR_VL);         /* f(y_t|I(t-1)), density of y_t given past
                                  information:  This is weighted average of
                                  2^5 conditional densities   */

   LIK=-1*LN(PR_VAL);

   PRO_=PR_VL/PR_VAL; /* Pr[S_t,S_{t-1},S_{t-2},S_{t-3},S_{t-4} | I(t-1),y_t]*/
                      /* Updating of prob. with new information, y_t   */


   PROB__T=PRO_[1:DMNSION/2,1]+PRO_[DMNSION/2+1:DMNSION,1];
                       /* Integrate out S_{t-4}: then you get
                          Pr[S_t, S_{t-1},S_{t-2},S_{t-3}| Y_t]  */
                       /*   2^4x1*/

   PROB__=VECR(PROB__T~PROB__T);
                       /*  2^5x1  */

LIKV = LIKV+LIK;
J_ITER = J_ITER+1;
ENDO;

               LOCATE 2,35;"LIKV=";;LIKV;
RETP(LIKV);
ENDP;
@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@

@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@


PROC (2) = FILTER(PRMTR1);
local prmtr, ppr,qpr,prob__0,prob__1,QQ, lik, j_iter,  pr__0_l,pr__1_l,
      F_cast, var_L,pr_vl, pr_val,likv,phi,PSIC,PSIX,
      vecp,st,st1,st2,st3,st4,ST5,ST6,ST7,ST8,ST9,ST10,ST11,ST12,ST13,
      pr_tr,pr_trf1,pr_trf,prob__t,prob__,pro_,j,psi1,psi2,var_c,
      delta0,DELTA1,MU0,MU1,st_k,st_k1,st_k2,st_k3,st_k4,
      f_cast1,f_cast2,PR_VL1,PR_VL2,pr_trf7,pr_trf0,
      PR_TRF2,PR_TRF3,PR_TRF4,PR_TRF5,PR_TRF6,psiL,PR_STL0,PR_STT0,
      TMPRY1,TMPRY2,SM_PRL,TMP_P0,SM_PR0,JJJ,MU_MAT,D_MAT,FLT_PRN,
      F1,F2,TMP00,TMP0,SM_PR00,prob_dd,VAR,TMP,OUT_MAT,A,EN;

PRMTR=TRANS(PRMTR1);

LOCATE 15,1; PRMTR';

           PPR=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI=PRMTR[3:6,1];
           VAR=PRMTR[7,1]^2;
           MU0=PRMTR[8,1];  @ recession, vs. boom@
           MU1=PRMTR[9,1];  @ recession, vs. boom@


           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           MU_MAT=ST_MAT*MU1 + (ONES(DMNSION,NO_ST)-ST_MAT)*MU0;

        /* FOR UNCONDITIONAL PROBABILITIES */

           A = (eye(2)-pr_tr)|ones(1,2);
           EN=(0|0|1);
           PROB__T = INV(A'A)*A'EN;  @PR[S_t=0]|PR[S_t=1], 2x1 UNCONDITIONAL
                                                       PROBABILITIES@


           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;
           PR_TRF =PR_TRF2|PR_TRF2;

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF0;      @2^2 x 1@
           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF1;      @2^3 x 1@
           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF2;      @2^4 x 1@
           PROB__=VECR(PROB__T~PROB__T);                @2^5 x 1@

  PR_STT0=ZEROS(T,1);  @WILL SAVE Pr[S_t=0|Y_{t}@
  PR_STL0=ZEROS(T,1);  @WILL SAVE Pr[S_t=0|Y_{t-1}@

  LIKV=0.0;
  J_ITER=1;
  DO UNTIL J_ITER>T;


      F_CAST1=(YY[J_ITER,1]-X_MAT[J_ITER,.]*PHI)*ONES(DMNSION,1)
                     -(MU_MAT[.,5]  - MU_MAT[.,4]*PHI[1,1]
                                    - MU_MAT[.,3]*PHI[2,1]
                                    - MU_MAT[.,2]*PHI[3,1]
                                    - MU_MAT[.,1]*PHI[4,1] );

     VAR_L=VAR*ONES(DMNSION,1);

     PROB_DD=PR_TRF .* PROB__;

@-----------------------------------------------------@
         TMP=PROB_DD;
         TMP=TMP[1:16]+TMP[17:32];
         TMP=TMP[1:8]+TMP[9:16];
         TMP=TMP[1:4]+TMP[5:8];
         TMP=TMP[1:2]+TMP[3:4];

    PR_STL0[J_ITER,1]=TMP[1,1];          @Pr[S_t=0|Y_t]@
@------------------------------------------------------@

     PR_VL=(1./SQRT(2.*PI.*VAR_L)).*EXP(-0.5*F_CAST1.*F_CAST1./VAR_L).*PROB_DD;

                      @PR[S_t,S_{T-1},S_{T-2},S_{T-3},S_{T-4},Y_t|Y_{t-1}]@

     PR_VAL=SUMC(PR_VL);   @f(y_t| Y_{t-1})@
     LIK=-1*LN(PR_VAL);

     PRO_=PR_VL/PR_VAL;    @Pr[S_t, S_{t-1},S_{t-2},S_{t-3},S_{t-4} | Y_t]@


@-------------------------------------------------------@
         TMP=PRO_;
         TMP=TMP[1:16]+TMP[17:32];
         TMP=TMP[1:8]+TMP[9:16];
         TMP=TMP[1:4]+TMP[5:8];
         TMP=TMP[1:2]+TMP[3:4];

    PR_STT0[J_ITER,1]=TMP[1,1];           @Pr[S_t=0|Y_t]@
@-------------------------------------------------------@

   PROB__T=PRO_[1:DMNSION/2,1]+PRO_[DMNSION/2+1:DMNSION,1];
                          @Pr[S_t, S_{t-1},S_{t-2},S_{t-3}| Y_t]@

   PROB__=VECR(PROB__T~PROB__T);


J_ITER = J_ITER+1;
ENDO;

RETP(PR_STT0,PR_STL0);
ENDP;
@======================================================================@
@======================================================================@


PROC SMOOTH(pr_tt0,pr_tl0);

         @pr_TT0 contains Pr[S_t|Y_t]@
         @pr_TL0 contains Pr[S_t|Y_{t-1}]@


local ppr, qpr, pr_sm0,pr_sm1, j_iter,pr_sm00,pr_sm01,pr_sm10,pr_sm11,
      pr_tt1,pr_tl1;

           PPR=PRM_fnl[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRM_fnl[2,1];    @Pr[St=0/St-1=0]@

           pr_tt1=1-pr_tt0;
           pr_tl1=1-pr_tl0;

           pr_sm0=pr_tt0;     @ pr_sm0 will contain Pr[S_t|Y_T]@
           pr_sm1=pr_tt1;


j_iter=T-1;
do until j_iter < 1;

    @The followings are P[S_t, S_t+1|Y_T] @

   pr_sm00=pr_sm0[j_iter+1,1]*qpr*    pr_tt0[j_iter,1]/ pr_tl0[j_iter+1,1];

   pr_sm01=pr_sm1[j_iter+1,1]*(1-qpr)*pr_tt0[j_iter,1]/ pr_tl1[j_iter+1,1];

   pr_sm10=pr_sm0[j_iter+1,1]*(1-ppr)*pr_tt1[j_iter,1]/ pr_tl0[j_iter+1,1];

   pr_sm11=pr_sm1[j_iter+1,1]*ppr*    pr_tt1[j_iter,1]/ pr_tl1[j_iter+1,1];

   pr_sm0[j_iter,1]=pr_sm00+pr_sm01;
   pr_sm1[j_iter,1]=pr_sm10+pr_sm11;

j_iter=j_iter -1;
endo;


RETP(pr_sm0); @This proc returns Pr[S_t=0|Y_T]@
endp;



@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@



PROC TRANS(c0); @ constraining values of reg. coeff.@
         local c1,m,u,d1,d2,d3,d4,d5,d6;

         c1=c0;

   c1[1:2,.]=exp(c0[1:2,.])./ (1+exp(c0[1:2,.]));


retp(c1);
endp;