/*
========================================================================
MODEL:  FILARDO'S(1994, JBES) MARKOV-SWITCHING MODEL WITH TIME-VARYING
                              TRANSITION PROBABILITIES


         (y_t-mu_{s_t})= phi_1*(y_{t-1}-mu_{s_{t-1}})+...
                        +phi_4*(y_{t-4}-mu_{s_{t-4}}) + e_t

                 e_t -- i.i.d. N(0,sigma^2)
         mu_{st} = (1-S_t)*mu_0 + S_t* mu_1

        Pr[S_t=0|S_{t-1}=0]=q_t=
                exp(q_0 + z_{t-1}*q_1)/(1+exp(q_0 + z_{t-1}*q_1))
        Pr[S_t=1|S_{t-1}=1]=p_t=
                    exp(p_0 + z_{t-1}*p_1)/(1+exp(p_0 + z_{t-1}*p_1))

        NOTE: i) z_t should be stationary.
              ii) z_t should be in deviation from the mean.
          iii) The time-varying transition probabilities may be
               functions of more than one variables.
              ========================================


WRITTEN BY CHANG-JIN KIM,        based on FILARDO (1994)
           DEPT. OF ECONOMICS
           KOREA UNIVERSITY
           SEOUL, 136-701, KOREA
       cjkim@.korea.ac.kr
=========================================================================
*/

new;
library optmum,PGRAPH;

format /m1 /rd 9,5;
load data[520,3]=filardo.prn;  @MONTH, IP, LEADING INDEX, 48:01-91:04@

yy_d=(ln(data[2:520,2])-ln(data[1:519,2]))*100; @1960:2--@
                    @first difference of log of IP@

YY_D[1:143,1]=YY_D[1:143,1]*STDC(YY_D[144:519,1])/STDC(YY_D[1:143,1]);
                    @NOTE: pre-1960 observations are deflated by the
                           ratio of the subsamples' standard deviations
                           Refer to Filardo (1994), page 302 @

yy_ld=(ln(data[2:520,3])-ln(data[1:519,3]))*100; @1960:2--@
                    @first difference of log of leading index@

yy_ld=yy_ld-meanc(yy_ld);  @in deviation from the mean@

t0=rows(yy_d);


LAG_AR=4;
NO_ST=lag_ar+1; @ NUMBER OF STATES TO BE CONSIDERED@
DMNSION=2^NO_ST;

output file=filardo.out reset;     OUTPUT OFF;


st_mat=zeros(DMNSION,NO_ST);

       j=1;
       st4=0;do until st4>1;
       st3=0; do until st3>1;
       st2=0; do until st2>1;
       st1=0; do until st1>1;
       st=0; do until st>1;

      st_mat[j,.]=st4~st3~st2~st1~st;

    j=j+1;
    st=st+1;endo;
    st1=st1+1;endo;
    st2=st2+1;endo;
    st3=st3+1;endo;
    st4=st4+1;endo;

yy=yy_d[2:t0,1];
z_mat=yy_ld[1:t0-1,1];

T=rows(yy);



@================= Initialize Global Variables======1959.1-89.9======@

           START=1;  @1952:4.....@

PRMTR_IN={
  4.35966   1.64982   0.18947   0.07933   0.11094   0.12226   0.69596
 -0.86585   0.51733   1.77043  -0.99472 };

PRMTR_IN=PRMTR_IN';

@ Maximum Likelihood Estimation @
@==================================================@

{xout,fout,gout,cout}=optmum(&lik_fcn,PRMTR_in);

PRM_FNL=TRANS(xout);             @ Estimated  coefficients, constrained@
output on;

"==FINAL OUTPUT========================================================";

"initial values of prmtr is";
                  trans(prmtr_in)';
"==============================================================";
"code is--------";;cout;
"likelihood value is ";; -1*fout;
"Estimated parameters are:";
prm_fnl'; " ";
xout';
output off;


"Calculating Hessian..... Please be patient!!!!";
                  hout0=hessp(&lik_fcn,xout);
                  hout=inv(hout0);

grdn_fnl=gradp(&TRANS,xout);
Hsn_fnl=grdn_fnl*hout*grdn_fnl';
SD_fnl =sqrt(diag(Hsn_fnl)); @Standard errors of the estimated coefficients@

output on;
"Standard errors of parameters are:"; sd_fnl';
"===============================================================";
output off;



{pr_tt0,pr_tl0}=FILTER(XOUT);  @Pr[S_t=0|Y_t] and Pr[S_t=0|Y_{t-1}]@

xy(seqa(1,1,rows(pr_tt0)),pr_tt0);


end;

@ END OF MAIN PROGRAM @




@========================================================================@
@========================================================================@
PROC LIK_FCN(PRMTR1);
local prmtr, ppr,qpr,prob__0,prob__1,QQ, lik, j_iter,  pr__0_l,pr__1_l,
      F_cast, var_L,pr_vl, pr_val,likv,phi,PSIC,PSIX,
      vecp,st,st1,st2,st3,st4,ST5,ST6,ST7,ST8,ST9,ST10,ST11,ST12,ST13,
      pr_tr,pr_trf1,pr_trf,prob__t,prob__,pro_,pr_vl,j,psi1,psi2,var_c,
      delta0,DELTA1,MU0,MU1,st_k,st_k1,st_k2,st_k3,st_k4,
      f_cast1,f_cast2,PR_VL1,PR_VL2,pr_trf7,pr_trf0,
      PR_TRF2,PR_TRF3,PR_TRF4,PR_TRF5,PR_TRF6,psic,psiL,
      TMPRY1,TMPRY2,SM_PRL,TMP_P0,SM_PR0,JJJ,MU_MAT,D_MAT,FLT_PRN,
      F1,F2,TMP00,TMP0,SM_PR0,SM_PR00,prob_dd,VAR,A,EN,Q0,Q1,
      P0,P1;

PRMTR=TRANS(PRMTR1);

LOCATE 15,1; PRMTR';

/* DEFINE PARAMETERS */

           P0=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           Q0=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI=PRMTR[3:6,1];
           VAR=PRMTR[7,1]^2;
           MU0=PRMTR[8,1];  @ recession, vs. boom@
           MU1=PRMTR[9,1];  @ recession, vs. boom@

           P1=PRMTR[10,1];
           Q1=PRMTR[11,1];

           MU_MAT=ST_MAT*MU1 + (ONES(DMNSION,NO_ST)-ST_MAT)*MU0;


/* INITIALIZING THE FILTER                             */
@-------------------------------------------------------@
@(1)----------------------------------------@
       PPR=EXP(P0+P1*z_mat[1,1])/
               (1+EXP(P0+P1*z_mat[1,1]));

           QPR=EXP(Q0+Q1*z_mat[1,1])/
               (1+EXP(Q0+Q1*z_mat[1,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           A = (eye(2)-pr_tr)|ones(1,2);
           EN=(0|0|1);
           PROB__T = INV(A'A)*A'EN;  @PR[S_t=0]|PR[S_t=1],
                                  2x1 steady-state PROBABILITIES@

@(2)------------------------------------@

           PPR=EXP(P0+P1*z_mat[2,1])/
               (1+EXP(P0+P1*z_mat[2,1]));
           QPR=EXP(Q0+Q1*z_mat[2,1])/
               (1+EXP(Q0+Q1*z_mat[2,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF0;
                                                           @2^2@

@ (3)-------------------------------------- @

           PPR=EXP(P0+P1*z_mat[3,1])/
               (1+EXP(P0+P1*z_mat[3,1]));
           QPR=EXP(Q0+Q1*z_mat[3,1])/
               (1+EXP(Q0+Q1*z_mat[3,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF1;
                                                           @2^3@

@ (4)-------------------------------------- @

           PPR=EXP(P0+P1*z_mat[4,1])/
               (1+EXP(P0+P1*z_mat[4,1]));
           QPR=EXP(Q0+Q1*z_mat[4,1])/
               (1+EXP(Q0+Q1*z_mat[4,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF2;
                                                           @2^4@

@(5)--------------------------------------@

            PROB__=VECR(PROB__T~PROB__T);
                                                           @ 2^5@

@==================================================================@
/* START ITERATION................................................*/

  LIKV=0.0;
  J_ITER=5;
  DO UNTIL J_ITER>T;

           QPR=EXP(Q0+Q1*Z_MAT[J_ITER,1])/
              (1+EXP(Q0+Q1*Z_MAT[J_ITER,1]));

           PPR=EXP(P0+P1*Z_MAT[J_ITER,1])/
              (1+EXP(P0+P1*Z_MAT[J_ITER,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;
           PR_TRF =PR_TRF2|PR_TRF2;


   F_CAST1=(YY[J_ITER,1]-REV(YY[J_ITER-4:J_ITER-1,1])'*PHI)*ONES(DMNSION,1)
               -(MU_MAT[.,5]  - MU_MAT[.,4]*PHI[1,1]
                              - MU_MAT[.,3]*PHI[2,1]
                              - MU_MAT[.,2]*PHI[3,1]
                              - MU_MAT[.,1]*PHI[4,1] );          /* 2^5x1 */

   VAR_L=VAR*ONES(DMNSION,1);                                    /* 2^5x1 */

   PROB_DD=PR_TRF .* PROB__;
                    /* Pr[S_t,S_{t-1},S_{t-2},S_{t-3},S_{t-4} | I(t-1)]*/

   PR_VL=(1./SQRT(2.*PI.*VAR_L)).*EXP(-0.5*F_CAST1.*F_CAST1./VAR_L).*PROB_DD;
                                                                 /* 2^5x1 */
                               /* Joint density of y_t,S_t,..,S_{t-4} given
                                  past information : */

   PR_VAL=SUMC(PR_VL);         /* f(y_t|I(t-1)), density of y_t given past
                                  information:  This is weighted average of
                                  2^5 conditional densities   */

   LIK=-1*LN(PR_VAL);

   PRO_=PR_VL/PR_VAL; /* Pr[S_t,S_{t-1},S_{t-2},S_{t-3},S_{t-4} | I(t-1),y_t]*/
                      /* Updating of prob. with new information, y_t   */


   PROB__T=PRO_[1:DMNSION/2,1]+PRO_[DMNSION/2+1:DMNSION,1];
                       /* Integrate out S_{t-4}: then you get
                          Pr[S_t, S_{t-1},S_{t-2},S_{t-3}| Y_t]  */
                       /*   2^4x1*/

   PROB__=VECR(PROB__T~PROB__T);
                       /*  2^5x1  */

LIKV = LIKV+LIK;
J_ITER = J_ITER+1;
ENDO;

               LOCATE 2,35;"LIKV=";;LIKV;
RETP(LIKV);
ENDP;
@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@

@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@


PROC (2) = FILTER(PRMTR1);
local prmtr, ppr,qpr,prob__0,prob__1,QQ, lik, j_iter,  pr__0_l,pr__1_l,
      F_cast, var_L,pr_vl, pr_val,likv,phi,PSIC,PSIX,
      vecp,st,st1,st2,st3,st4,ST5,ST6,ST7,ST8,ST9,ST10,ST11,ST12,ST13,
      pr_tr,pr_trf1,pr_trf,prob__t,prob__,pro_,pr_vl,j,psi1,psi2,var_c,
      delta0,DELTA1,MU0,MU1,st_k,st_k1,st_k2,st_k3,st_k4,
      f_cast1,f_cast2,PR_VL1,PR_VL2,pr_trf7,pr_trf0,
      PR_TRF2,PR_TRF3,PR_TRF4,PR_TRF5,PR_TRF6,psic,psiL,PR_STL0,PR_STT0,
      TMPRY1,TMPRY2,SM_PRL,TMP_P0,SM_PR0,JJJ,MU_MAT,D_MAT,FLT_PRN,
      F1,F2,TMP00,TMP0,SM_PR0,SM_PR00,prob_dd,VAR,TMP,OUT_MAT,A,EN,
      Q0,Q1,P0,P1;

PRMTR=TRANS(PRMTR1);

LOCATE 15,1; PRMTR';


           P0=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           Q0=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI=PRMTR[3:6,1];
           VAR=PRMTR[7,1]^2;
           MU0=PRMTR[8,1];  @ recession, vs. boom@
           MU1=PRMTR[9,1];  @ recession, vs. boom@

       P1=PRMTR[10,1];
       Q1=PRMTR[11,1];

       MU_MAT=ST_MAT*MU1 + (ONES(DMNSION,NO_ST)-ST_MAT)*MU0;


/* INITIALIZING THE FILTER                             */
@-------------------------------------------------------@
@(1)----------------------------------------@
       PPR=EXP(P0+P1*z_mat[1,1])/
               (1+EXP(P0+P1*z_mat[1,1]));

           QPR=EXP(Q0+Q1*z_mat[1,1])/
               (1+EXP(Q0+Q1*z_mat[1,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           A = (eye(2)-pr_tr)|ones(1,2);
           EN=(0|0|1);
           PROB__T = INV(A'A)*A'EN;  @PR[S_t=0]|PR[S_t=1],
                                  2x1 steady-state PROBABILITIES@

@(2)------------------------------------@

           PPR=EXP(P0+P1*z_mat[2,1])/
               (1+EXP(P0+P1*z_mat[2,1]));
           QPR=EXP(Q0+Q1*z_mat[2,1])/
               (1+EXP(Q0+Q1*z_mat[2,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF0;
                                                           @2^2@

@ (3)-------------------------------------- @

           PPR=EXP(P0+P1*z_mat[3,1])/
               (1+EXP(P0+P1*z_mat[3,1]));
           QPR=EXP(Q0+Q1*z_mat[3,1])/
               (1+EXP(Q0+Q1*z_mat[3,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF1;
                                                           @2^3@

@ (4)-------------------------------------- @

           PPR=EXP(P0+P1*z_mat[4,1])/
               (1+EXP(P0+P1*z_mat[4,1]));
           QPR=EXP(Q0+Q1*z_mat[4,1])/
               (1+EXP(Q0+Q1*z_mat[4,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;

           PROB__T=VECR(PROB__T~PROB__T).*PR_TRF2;
                                                           @2^4@

@(5)--------------------------------------@

            PROB__=VECR(PROB__T~PROB__T);
                                                           @ 2^5@

@==================================================================@
/* START ITERATION................................................*/

  PR_STT0=ZEROS(T,1);  @WILL SAVE Pr[S_t=0|Y_{t}@
  PR_STL0=ZEROS(T,1);  @WILL SAVE Pr[S_t=0|Y_{t-1}@

  LIKV=0.0;
  J_ITER=5;
  DO UNTIL J_ITER>T;

           QPR=EXP(Q0+Q1*Z_MAT[J_ITER,1])/
              (1+EXP(Q0+Q1*Z_MAT[J_ITER,1]));

           PPR=EXP(P0+P1*Z_MAT[J_ITER,1])/
              (1+EXP(P0+P1*Z_MAT[J_ITER,1]));

           PR_TR=(QPR~    (1-PPR))|
                 ((1-QPR)~  PPR);

           PR_TRF0=VEC(PR_TR);
           PR_TRF1=PR_TRF0|PR_TRF0;
           PR_TRF2=PR_TRF1|PR_TRF1;
           PR_TRF =PR_TRF2|PR_TRF2;

      F_CAST1=(YY[J_ITER,1]-REV(YY[J_ITER-4:J_ITER-1,1])'*PHI)*ONES(DMNSION,1)
               -(MU_MAT[.,5]  - MU_MAT[.,4]*PHI[1,1]
                              - MU_MAT[.,3]*PHI[2,1]
                              - MU_MAT[.,2]*PHI[3,1]
                              - MU_MAT[.,1]*PHI[4,1] );          /* 2^5x1 */

     VAR_L=VAR*ONES(DMNSION,1);

     PROB_DD=PR_TRF .* PROB__;

@-----------------------------------------------------@
         TMP=PROB_DD;
         TMP=TMP[1:16]+TMP[17:32];
         TMP=TMP[1:8]+TMP[9:16];
         TMP=TMP[1:4]+TMP[5:8];
         TMP=TMP[1:2]+TMP[3:4];

    PR_STL0[J_ITER,1]=TMP[1,1];          @Pr[S_t=0|Y_t]@
@------------------------------------------------------@

     PR_VL=(1./SQRT(2.*PI.*VAR_L)).*EXP(-0.5*F_CAST1.*F_CAST1./VAR_L).*PROB_DD;

                      @PR[S_t,S_{T-1},S_{T-2},S_{T-3},S_{T-4},Y_t|Y_{t-1}]@

     PR_VAL=SUMC(PR_VL);   @f(y_t| Y_{t-1})@
     LIK=-1*LN(PR_VAL);

     PRO_=PR_VL/PR_VAL;    @Pr[S_t, S_{t-1},S_{t-2},S_{t-3},S_{t-4} | Y_t]@


@-------------------------------------------------------@
         TMP=PRO_;
         TMP=TMP[1:16]+TMP[17:32];
         TMP=TMP[1:8]+TMP[9:16];
         TMP=TMP[1:4]+TMP[5:8];
         TMP=TMP[1:2]+TMP[3:4];

    PR_STT0[J_ITER,1]=TMP[1,1];           @Pr[S_t=0|Y_t]@
@-------------------------------------------------------@

   PROB__T=PRO_[1:DMNSION/2,1]+PRO_[DMNSION/2+1:DMNSION,1];
                          @Pr[S_t, S_{t-1},S_{t-2},S_{t-3}| Y_t]@

   PROB__=VECR(PROB__T~PROB__T);


J_ITER = J_ITER+1;
ENDO;

RETP(PR_STT0,PR_STL0);
ENDP;
@======================================================================@



PROC TRANS(c0); @ constraining values of reg. coeff.@
         local c1,m,u,d1,d2,d3,d4,d5,d6;

         c1=c0;

retp(c1);
endp;

