/*===================================================================
  A MODIFIED, SIMPLER, VERSION OF THE PROGRAM for
  "DYNAMIC LINEAR MODELS WITH MARKOV-SWITCHING"
  KIM, CHANG-JIN (1994), JOURNAL OF ECONOMETRICS, 60, 1-22.

  (IF YOU WANT THE ORIGINAL VERSION OF THE PROGRAM THAT EXACTLY
   REPLICATES THE RESULTS IN THE PAPER, PLEASE CONTACT ME.)

  In this program, initial values of the state-vector (beta_00) and
  its covariance matrix are given by their steady-state values.
  The log likelihood value was evaluated  starting from the 23rd 
  observation (1952.2).



  WRITTEN BY CHANG-JIN KIM,
  DEPT. OF ECONOMICS
  KOREA UNIVERSITY,
  SEOUL, 136-701, KOREA

  E-MAIL: cjkim@kuccnx.korea.ac.kr

""; ""; "";
"     >>>>>>>>>>>>>>>>>>>>STATE-SPACE REPRESENTATION OF  >>>>>>>>>>>";
"     >>>>>>>>>>>>>>>>>>>>LAM'S (1991) GENERALIZED       >>>>>>>>>>>";
"     ====================HAMILTON        MODEL          ===========";

"                   y_t = z_t + n_t                                 ";
"                    n_t =n_t-1  +mu0 + MU_1* S_t                   ";
"                  z_t = phi_1 * z_t-1  + phi_2 *z_t-2 + w_t        ";
"               Pr[S_t=1|S_t-1=1]=p;    Pr[S_t=0|S_t-1=1]=1-p       ";
"               Pr[S_t=1|S_t-1=0]=1-q;  Pr[S_t=1|S_t-1=1]=q         ";
=======================================================================*/
/*===============================================================*/
/* This program is written and Copyright (c) 1998 by Chang-Jin Kim
Please refer to http://weber.u.washington.edu/~cnelson/SSMARKOV.html 
for conditions of use. */
/*===============================================================*/

new;
library optmum,PGRAPH;
output file=KIM_JE.ouT reset;  OUTPUT OFF;


format /m1 /rd 9,4;

          load rgnp[160,2]=KIM_JE.PRN;   @ Lam's Data Set@
          qtr=rgnp[.,1];
          rgnp=rgnp[.,2];                         @N_ob== 172@

          lngnp =ln(rgnp)*100;
          T0=rows(lngnp);


          yy =lngnp[2:T0,1]-lngnp[1:T0-1,1]; @N_ob is 159@
                                             @1947.2 -- 1986.4@



yy=yy[1:151,1];  T=rows(yy);  @1952.2--1984.4@

lngnp=lngnp[24:152,1];

@================= Initialize Global Variables============@


           START=23;  @1952:2.....@

           PRMTR_NO=7;   @ >>Number of parameters to be estimated. >>@
           dmnsion=2;        @ >> dimension of transition equation >>@

prmtr_in={3.5 0.0  0.5 1  0.7  -0.4 1.7};

prmtr_in=prmtr_in';

@ Maximum Likelihood Estimation @
@==================================================@

{xout,fout,gout,cout}=optmum(&lik_fcn,PRMTR_IN);

PRM_FNL=TRANS(xout);             @ Estimated  coefficients, constrained@
output on;

"==FINAL OUTPUT========================================================";

"likelihood value is ";; -1*fout;
"Estimated parameters are:";
prm_fnl';
output off;


"Calculating Hessian..... Please be patient!!!!";
                  hout0=hessp(&lik_fcn,xout);
                  hout=inv(hout0);

grdn_fnl=gradp(&TRANS,xout);
Hsn_fnl=grdn_fnl*hout*grdn_fnl';
SD_fnl =sqrt(diag(Hsn_fnl)); @Standard errors of the estimated coefficients@

output on;
"Standard errors of parameters are:"; sd_fnl;
"===============================================================";
output off;


{CYCLE,PR_TT0,PR_TL0} =FILTER(XOUT);

TREND=LNGNP-CYCLE;

output file=kim_je.dta reset; lngnp~cycle~trend; output off;
smooth0=SMOOTH(pr_tt0,pr_tl0); @Pr[S_t=0|Y_T], Smoothed probabilities@

XY(SEQA(1,1,ROWS(TREND)),TREND~LNGNP);
XY(SEQA(1,1,ROWS(SMOOTH0)),SMOOTH0);

end;

@ END OF MAIN PROGRAM @
@========================================================================@
@========================================================================@


PROC LIK_FCN(PRMTR1);

LOCAL PRMTR, PPR,QPR, PHI1,PHI2, VAR_W, MU0,MU1,R,Q,H,F,B_LL0,B_LL1,
      VECP_LL,P_LL0,P_LL1,PROB__1,PROB__0,LIKV,J_ITER,B_TL00,B_TL01,
      B_TL10,B_TL11,P_TL00,P_TL01,P_TL10,P_TL11,F_CAST00,F_CAST01,
      F_CAST10,F_CAST11,SS00,SS01,SS10,SS11,B_TT00,B_TT01,B_TT10,
      B_TT11,P_TT00,P_TT01,P_TT10,P_TT11,PR_VL00,PR_VL01,PR_VL10,PR_VL11,
      PR_VAL,PRO_00,PRO_01,PRO_10,PRO_11;

EXTERNAL PROC TRANS, V_PROB;


           PRMTR=TRANS(PRMTR1);

           LOCATE 15,1;  PRMTR';

           PPR=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI1=PRMTR[3,1]; PHI2=PRMTR[4,1];

           VAR_W=sqrt(PRMTR[5,1]^2);

           MU0=PRMTR[6,1];
           MU1=MU0+ PRMTR[7,1];

           R=0;
           Q=(VAR_W^2~ 0)|( 0~ 0);
           H=(1~ -1);
           F =(phi1~ phi2 )|
              ( 1  ~  0   );


/*  NOTE:  KIM'S FILTER = HAMILTON FILTER + KALMAN FILTER               */
/*  INITIALIZING THE FILTER: FOR BOTH KALMAN FILTER AND HAMILTON FILTER */

         B_LL0=0|0;
         B_LL1=0|0;

         VECP_LL=INV(EYE(4) - F.*.F)*vec(Q);

         P_LL0=VECP_LL[1:2,1]~VECP_LL[3:4,1];
         P_LL1=P_LL0;

         PROB__1=(1-QPR)/(2-PPR-QPR);   @Pr[S_0=1/Y_0], STEADY STATE PROB.@
         PROB__0=1-PROB__1  ;           @Pr[S_0=0/Y_0], STEADY STATE PROB @


/*  START ITERATION */
/*===================================================================== */

       LIKV=0.0;
       J_ITER = 1;
       DO UNTIL J_ITER>T;

       /* KALMAN FILTER */
       /*==============================================================*/

             B_TL00 = F * B_LL0;  @WHEN S_{t-1}=0, S_{t}=0@
             B_TL01 = F * B_LL0;  @WHEN S_{t-1}=0, S_{t}=1@
             B_TL10 = F * B_LL1;  @WHEN S_{t-1}=1, S_{t}=0@
             B_TL11 = F * B_LL1;  @WHEN S_{t-1}=1, S_{t}=1@

             P_TL00 = F * P_LL0 * F' + Q;
             P_TL01 = F * P_LL0 * F' + Q;
             P_TL10 = F * P_LL1 * F' + Q;
             P_TL11 = F * P_LL1 * F' + Q;

             F_CAST00= yy[J_ITER,.]- H * B_TL00- MU0;
             F_CAST01= yy[J_ITER,.]- H * B_TL01- MU1;
             F_CAST10= yy[J_ITER,.]- H * B_TL10- MU0;
             F_CAST11= yy[J_ITER,.]- H * B_TL11- MU1;


             SS00= H * P_TL00 * H' +R; @VARIANCE OF FORECAST ERROR@
             SS01= H*  P_TL01 * H' +R;
             SS10= H * P_TL10 * H' +R;
             SS11= H * P_TL11 * H' +R;

             B_TT00 = B_TL00 + (P_TL00 * H' / SS00) * F_CAST00;
             B_TT01 = B_TL01 + (P_TL01 * H' / SS01) * F_CAST01;
             B_TT10 = B_TL10 + (P_TL10 * H' / SS10) * F_CAST10;
             B_TT11 = B_TL11 + (P_TL11 * H' / SS11) * F_CAST11;

             P_TT00 = (EYE(2) - (P_TL00 * H' / SS00) * H ) * P_TL00;
             P_TT01 = (EYE(2) - (P_TL01 * H' / SS01) * H ) * P_TL01;
             P_TT10 = (EYE(2) - (P_TL10 * H' / SS10) * H ) * P_TL10;
             P_TT11 = (EYE(2) - (P_TL11 * H' / SS11) * H ) * P_TL11;


       /* HAMILTON FILTER */
       /*==============================================================*/


             PR_VL00=V_PROB(F_CAST00,SS00)*  QPR*PROB__0;    @Pr[St,Yt/Yt-1]@
             PR_VL01=V_PROB(F_CAST01,SS01)*(1-QPR)*PROB__0;
             PR_VL10=V_PROB(F_CAST10,SS10)*(1-PPR)*PROB__1;
             PR_VL11=V_PROB(F_CAST11,SS11)*    PPR*PROB__1;
                                          @CONDITIONAL DENSITY TIMES WEIGHT@

             PR_VAL=PR_VL00+PR_VL01+PR_VL10+PR_VL11;
                                  @WEIGHTED AVERAGE OF CONDITIONAL DENSITIES:
                                   f(y_t|Y_{t-1})@

             PRO_00=PR_VL00/PR_VAL;       @Pr[St,St-1/Yt]@
             PRO_01=PR_VL01/PR_VAL;
             PRO_10=PR_VL10/PR_VAL;
             PRO_11=PR_VL11/PR_VAL;

             PROB__0=PRO_00+PRO_10;       @Pr[St=0/Yt]@
             PROB__1=PRO_01+PRO_11;       @Pr[St=1/Yt]@


       /* COLLAPSING TERMS */
       /*==============================================================*/


              B_LL0=(PRO_00*B_TT00 + PRO_10*B_TT10)/PROB__0;
              B_LL1=(PRO_01*B_TT01 + PRO_11*B_TT11)/PROB__1;


              P_LL0=(PRO_00*(P_TT00+(B_LL0-B_TT00)*(B_LL0-B_TT00)')+
                     PRO_10*(P_TT10+(B_LL0-B_TT10)*(B_LL0-B_TT10)'))/PROB__0;

              P_LL1=(PRO_01*(P_TT01+(B_LL1-B_TT01)*(B_LL1-B_TT01)')+
                     PRO_11*(P_TT11+(B_LL1-B_TT11)*(B_LL1-B_TT11)'))/PROB__1;


              IF J_ITER < START; goto skip; endif;
              LIKV = LIKV -LN(PR_VAL);

                skip:
       J_ITER = J_ITER+1;
       ENDO;

/*  END ITERATION    */
/*===================*/


RETP(LIKV);
ENDP;



@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>@
PROC TRANS(coef0); @ constraining values of reg. coeff.@
         local coef1,m,u,XX6,XX7;

      coef1=coef0;
      coef1[1:2,1]=EXP(COEF0[1:2,1]) ./ (1 + EXP(coef0[1:2,1]));

      XX6 = coef0[3,1]/(1 + abs(coef0[3,1]));
      XX7 = coef0[4,1]/(1 + abs(coef0[4,1]));

      coef1[3,1] = XX6 + XX7 ;
      coef1[4,1] =-1*XX6*XX7 ;

retp(coef1);
endp;


@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>@
PROC V_PROB(EV, HE);      @ CALCULATES    Pr[Yt/St,Yt-1]@

         LOCAL VAL;
         VAL=(1/SQRT(2*PI*HE))*EXP(-0.5*EV*EV/HE);

RETP(VAL);
ENDP;
@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>@


PROC (3) = FILTER(PRMTR1);

LOCAL PRMTR, PPR,QPR, PHI1,PHI2, VAR_W, MU0,MU1,R,Q,H,F,B_LL0,B_LL1,
      VECP_LL,P_LL0,P_LL1,PROB__1,PROB__0,LIKV,J_ITER,B_TL00,B_TL01,
      B_TL10,B_TL11,P_TL00,P_TL01,P_TL10,P_TL11,F_CAST00,F_CAST01,
      F_CAST10,F_CAST11,SS00,SS01,SS10,SS11,B_TT00,B_TT01,B_TT10,
      B_TT11,P_TT00,P_TT01,P_TT10,P_TT11,PR_VL00,PR_VL01,PR_VL10,PR_VL11,
      PR_VAL,PRO_00,PRO_01,PRO_10,PRO_11,B_TT,CYCLE_M,PR_TT0M,PR_TL0M;

EXTERNAL PROC TRANS, V_PROB;


           PRMTR=TRANS(PRMTR1);

           LOCATE 15,1;  PRMTR';

           PPR=PRMTR[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRMTR[2,1];    @Pr[St=0/St-1=0]@

           PHI1=PRMTR[3,1]; PHI2=PRMTR[4,1];

           VAR_W=sqrt(PRMTR[5,1]^2);

           MU0=PRMTR[6,1];
           MU1=MU0+ PRMTR[7,1];

           R=0;
           Q=(VAR_W^2~ 0)|( 0~ 0);
           H=(1~ -1);
           F =(phi1~ phi2 )|
              ( 1  ~  0   );

CYCLE_M=ZEROS(T,1);  @will store cyclical component: (1,1)element of B_TT@
PR_TT0M=ZEROS(T,1);  @will store Pr[S_t=0|Y_t]@
PR_TL0M=ZEROS(T,1);  @will store Pr[S_t=0|Y_{t-1}]@


/*  NOTE:  KIM'S FILTER = HAMILTON FILTER + KALMAN FILTER               */
/*  INITIALIZING THE FILTER: FOR BOTH KALMAN FILTER AND HAMILTON FILTER */

         B_LL0=0|0;
         B_LL1=0|0;

         VECP_LL=INV(EYE(4) - F.*.F)*vec(Q);

         P_LL0=VECP_LL[1:2,1]~VECP_LL[3:4,1];
         P_LL1=P_LL0;

         PROB__1=(1-QPR)/(2-PPR-QPR);   @Pr[S_0=1/Y_0], STEADY STATE PROB.@
         PROB__0=1-PROB__1  ;           @Pr[S_0=0/Y_0], STEADY STATE PROB @


/*  START ITERATION */
/*===================================================================== */

       LIKV=0.0;
       J_ITER = 1;
       DO UNTIL J_ITER>T;

PR_TL0M[J_ITER,1]= QPR*PROB__0 + (1-PPR)*PROB__1;     @Pr[St=0/Yt-1]@



       /* KALMAN FILTER */
       /*==============================================================*/

             B_TL00 = F * B_LL0;  @WHEN S_{t-1}=0, S_{t}=0@
             B_TL01 = F * B_LL0;  @WHEN S_{t-1}=0, S_{t}=1@
             B_TL10 = F * B_LL1;  @WHEN S_{t-1}=1, S_{t}=0@
             B_TL11 = F * B_LL1;  @WHEN S_{t-1}=1, S_{t}=1@

             P_TL00 = F * P_LL0 * F' + Q;
             P_TL01 = F * P_LL0 * F' + Q;
             P_TL10 = F * P_LL1 * F' + Q;
             P_TL11 = F * P_LL1 * F' + Q;

             F_CAST00= yy[J_ITER,.]- H * B_TL00- MU0;
             F_CAST01= yy[J_ITER,.]- H * B_TL01- MU1;
             F_CAST10= yy[J_ITER,.]- H * B_TL10- MU0;
             F_CAST11= yy[J_ITER,.]- H * B_TL11- MU1;


             SS00= H * P_TL00 * H' +R; @VARIANCE OF FORECAST ERROR@
             SS01= H*  P_TL01 * H' +R;
             SS10= H * P_TL10 * H' +R;
             SS11= H * P_TL11 * H' +R;

             B_TT00 = B_TL00 + (P_TL00 * H' / SS00) * F_CAST00;
             B_TT01 = B_TL01 + (P_TL01 * H' / SS01) * F_CAST01;
             B_TT10 = B_TL10 + (P_TL10 * H' / SS10) * F_CAST10;
             B_TT11 = B_TL11 + (P_TL11 * H' / SS11) * F_CAST11;

             P_TT00 = (EYE(2) - (P_TL00 * H' / SS00) * H ) * P_TL00;
             P_TT01 = (EYE(2) - (P_TL01 * H' / SS01) * H ) * P_TL01;
             P_TT10 = (EYE(2) - (P_TL10 * H' / SS10) * H ) * P_TL10;
             P_TT11 = (EYE(2) - (P_TL11 * H' / SS11) * H ) * P_TL11;


       /* HAMILTON FILTER */
       /*==============================================================*/


             PR_VL00=V_PROB(F_CAST00,SS00)*  QPR*PROB__0;    @Pr[St,Yt/Yt-1]@
             PR_VL01=V_PROB(F_CAST01,SS01)*(1-QPR)*PROB__0;
             PR_VL10=V_PROB(F_CAST10,SS10)*(1-PPR)*PROB__1;
             PR_VL11=V_PROB(F_CAST11,SS11)*    PPR*PROB__1;
                                          @CONDITIONAL DENSITY TIMES WEIGHT@

             PR_VAL=PR_VL00+PR_VL01+PR_VL10+PR_VL11;
                                  @WEIGHTED AVERAGE OF CONDITIONAL DENSITIES:
                                   f(y_t|Y_{t-1})@

             PRO_00=PR_VL00/PR_VAL;       @Pr[St,St-1/Yt]@
             PRO_01=PR_VL01/PR_VAL;
             PRO_10=PR_VL10/PR_VAL;
             PRO_11=PR_VL11/PR_VAL;

B_TT=B_TT00*PRO_00+B_TT01*PRO_01+B_TT10*PRO_10+B_TT11*PRO_11;
CYCLE_M[J_ITER,1]=B_TT[1,1];

             PROB__0=PRO_00+PRO_10;       @Pr[St=0/Yt]@
             PROB__1=PRO_01+PRO_11;       @Pr[St=1/Yt]@

PR_TT0M[J_ITER,1]=PROB__0;

       /* COLLAPSING TERMS */
       /*==============================================================*/


              B_LL0=(PRO_00*B_TT00 + PRO_10*B_TT10)/PROB__0;
              B_LL1=(PRO_01*B_TT01 + PRO_11*B_TT11)/PROB__1;


              P_LL0=(PRO_00*(P_TT00+(B_LL0-B_TT00)*(B_LL0-B_TT00)')+
                     PRO_10*(P_TT10+(B_LL0-B_TT10)*(B_LL0-B_TT10)'))/PROB__0;

              P_LL1=(PRO_01*(P_TT01+(B_LL1-B_TT01)*(B_LL1-B_TT01)')+
                     PRO_11*(P_TT11+(B_LL1-B_TT11)*(B_LL1-B_TT11)'))/PROB__1;


              IF J_ITER < START; goto skip; endif;
              LIKV = LIKV -LN(PR_VAL);

                skip:
       J_ITER = J_ITER+1;
       ENDO;

/*  END ITERATION    */
/*===================*/


RETP(CYCLE_M[START:T],PR_TT0M[START:T],PR_TL0M[START:T]);
ENDP;


@==========================================================@



PROC SMOOTH(pr_tt0,pr_tl0);

         @pr_TT0 contains Pr[S_t|Y_t]@
         @pr_TL0 contains Pr[S_t|Y_{t-1}]@


local ppr, qpr, pr_sm0,pr_sm1, j_iter,pr_sm00,pr_sm01,pr_sm10,pr_sm11,
      pr_tt1,pr_tl1,TT;

          TT=ROWS(PR_TT0);

           PPR=PRM_fnl[1,1];    @Pr[St=1/St-1=1]@
           QPR=PRM_fnl[2,1];    @Pr[St=0/St-1=0]@

           pr_tt1=1-pr_tt0;
           pr_tl1=1-pr_tl0;

           pr_sm0=pr_tt0;     @ pr_sm0 will contain Pr[S_t|Y_T]@
           pr_sm1=pr_tt1;


j_iter=TT-1;
do until j_iter < 1;

    @The followings are P[S_t, S_t+1|Y_T] @

   pr_sm00=pr_sm0[j_iter+1,1]*qpr*    pr_tt0[j_iter,1]/ pr_tl0[j_iter+1,1];

   pr_sm01=pr_sm1[j_iter+1,1]*(1-qpr)*pr_tt0[j_iter,1]/ pr_tl1[j_iter+1,1];

   pr_sm10=pr_sm0[j_iter+1,1]*(1-ppr)*pr_tt1[j_iter,1]/ pr_tl0[j_iter+1,1];

   pr_sm11=pr_sm1[j_iter+1,1]*ppr*    pr_tt1[j_iter,1]/ pr_tl1[j_iter+1,1];

   pr_sm0[j_iter,1]=pr_sm00+pr_sm01;
   pr_sm1[j_iter,1]=pr_sm10+pr_sm11;

j_iter=j_iter -1;
endo;


RETP(pr_sm0); @This proc returns Pr[S_t=0|Y_T]@
endp;



@++++++++++++++++++++++++++++++++++++++++++++++++++++++++@
