@>>>>>>>>>>>>>>GARCH MODEL FOR  STOCK RETURNS<<<<<<<@
@>>>>>>>>>>>>>>       Yt=  B +ut <<<<<<<<<<<<<<<<<<<<<<<<<@
@>>>>>>>>>>>>>>           ut--N(0,Ht)   <<<<<<<<<<<<<<<<<<<<<<<<<@
@>>>>>>>>>>>>>>   Ht=a0 + a1*ht-1 a2*ut-1^2  <<<<<<<@
@          parameters to estimate: B, a0, a1,a2         @
@                               (total number=4)                 @

@ NAME OF THE FILE:  GARCH.OPT @
/*   WRITTEN BY CHANG-JIN KIM, DEPT. OF ECON. KOREA UNIVERSITY 
cjkim@korea.ac.kr   */

new;
library optmum,pgraph;

format /m1 /rd 12,6;

format /m1 /rd 12,6;
load data[571,2]=turner3.prn; @month, s&p500 excess returns 47:2-94:09@

yy=data[156:571,2];  @1960:01--@
t=rows(yy);

output file=GARCH.out reset;

"=============================================================";
output off;

     @================= Initialize Global Variables============@




   @initial values of parameters @

     PRMTR_IN=  0.00279|  0.1252|   3.11965|   0.841;



@ Maximum Likelihood Estimation @

 {xout,fout,gout,cout}=optmum(&lik_fcn,PRMTR_IN);
                     @ prmtr estimates, -log lik value, Grandient, code@

"Calculating Hessian, and inverting..... Please be patient!!!!";

cov0=inv(hessp(&lik_fcn,xout));

grdn_fnl=gradp(&TRANS,xout);


prm_fnl=trans(xout);
cov_fnl=grdn_fnl*cov0*grdn_fnl';
SD_fnl =sqrt(diag(cov_fnl)); @Standard errors of the estimated coefficients@


SD_fnl =sqrt(diag(cov_fnl)); @Standard errors of the estimated coefficients@

output file=garch.out reset;

FORMAT /m1 /rd 9,4;

"=============================================================";
      "likelihood value is ";; -fout;
      "Estimated parameters are:";
      "(";; xout';;")";
       prm_fnl';
      " ";
      "Standard errors of parameters are:"; sd_fnl';
      "T-values are:"; prm_fnl'./sd_fnl';
"==============================================================";
output off;

con_ev=ht_out(xout);
xy(seqa(1,1,rows(con_ev)),con_ev[.,2]);

@========================================================================@



PROC LIK_FCN(PRMTR1);
       local aaa0,bbb,etl,et,evtr,evtr2,y00,yl1,yl2,
             zone,yl3,yl4,x00,xl1,xl2,xl3,xl4,lik,lik_raw,prmtr,
             HT_IN,aaa1,aaa2, indx,htl,ht;


   lik_raw=zeros(T,1);
   prmtr=trans(prmtr1);

   LOCATE 20,1; PRMTR';

    bbb=prmtr[1,1]; aaa0=prmtr[2,1];aaa1=prmtr[3,1];aaa2=prmtr[4,1];



    etl=YY[1,1]-BBB;
    htl=AAA0/(1-AAA1-AAA2);


indx=2;
do until indx>T;

          ht=aaa0 +aaa1*htl +aaa2*etl^2;
          et= yy[indx,1] - bbb;

          lik_raw[indx,1]= -0.5 * ln(2*pi*ht) - 0.5 * (et^2/ ht  );

          htl=ht;
          etl=et;

indx=indx+1;
endo;

          lik=-sumc(lik_raw[2:T,.]);

retp(lik);
endp;

@=================================@

PROC HT_OUT(PRMTR1);
       local aaa0,bbb,etl,et,evtr,evtr2,y00,yl1,yl2,
             zone,yl3,yl4,x00,xl1,xl2,xl3,xl4,lik,lik_raw,prmtr,
             HT_IN,aaa1,aaa2, indx,htl,ht,ht_mat;

    HT_MAT=ZEROS(T,2);

    prmtr=trans(prmtr1);

    bbb=prmtr[1,1]; aaa0=prmtr[2,1];aaa1=prmtr[3,1];aaa2=prmtr[4,1];


    etl=YY[1,1]-BBB;
    htl=AAA0/(1-AAA1-AAA2);

indx=2;
do until indx>T;

          ht=aaa0 +aaa1*htl +aaa2*etl^2;
          et= yy[indx,1] - bbb;

          ht_mat[indx,.]=et~ht;

          htl=ht;
          etl=et;

indx=indx+1;
endo;



retp(ht_mat[2:t,.]);
endp;

@=================================@


PROC TRANS(coef0); @ constraining values of reg. coeff.@
   local coef1,m,u;

   coef1=coef0;
   coef1[2,1]=coef0[2,1]^2;

   coef1[3,1]=COEF0[3,1]^2/(1+COEF0[3,1]^2+COEF0[4,1]^2);
   coef1[4,1]=COEF0[4,1]^2/(1+COEF0[3,1]^2+COEF0[4,1]^2);

   retp(coef1);
endp;


END;
