@===================================================================@
@          Model:   Y_t = X*beta + e_t,  e_t -- N(0,sigma^2)        @
@ MAXIMUM LIKELIHOOD ESTIMATION OF beta and  sigma (K+1 parameters) @
@===================================================================@
/* Chang-Jin Kim */

new;
library optmum;

     output file=mle_ar4.out reset; output off;


     format /m1 /rd 9,5;
     load data[195,1]=gdp4795.prn;  @Quarter, U.S. real GNP@
                              @1947.1 -- 1995.3 @

     yy=(ln(data[21:195,1])-ln(data[20:194,1]))*100;  @1952.II - 1995.3@
     T1=rows(yy);


     Y=YY[5:T1,1];
     X=ONES(t1-4,1)~YY[4:T1-1,1]~YY[3:T1-2,1]~YY[2:T1-3,1]~YY[1:T1-4,1];
     t=rows(y);


@----------INITIAL VALUES OF THE PARAMETERS@
prmtr_in=0.1|0|0|0|0|ln(0.5);

@----------MAXIMUM LIKELIHOOD ESTIMATION@

{xout,fout,gout,cout}=optmum(&like_v,prmtr_in);

PRM_FNL=TRANS(xout);             @ Estimated  coefficients, constrained@
output on;

"==FINAL OUTPUT========================================================";

"code is--------";;cout;
"likelihood value is ";; -1*fout;
"Estimated parameters are:";
prm_fnl'; " ";
xout';
output off;


"Calculating Hessian..... Please be patient!!!!";
                  hout0=hessp(&like_v,xout);
                  hout=inv(hout0);

grdn_fnl=gradfd(&TRANS,xout);
Hsn_fnl=grdn_fnl*hout*grdn_fnl';
SD_fnl =sqrt(diag(Hsn_fnl)); @Standard errors of the estimated coefficients@

output on;
"Standard errors of parameters are:"; sd_fnl';
"===============================================================";
output off;




end;

@============END OF THE MAIN PROGRAM=================================@

proc like_v(prmtr1);

local beta, sig2, et, lik_mat,jj,val,loglik,prmtr;

     prmtr=trans(prmtr1);

     beta=prmtr[1:5,1];
     sig2=prmtr[6,1];

     lik_mat=zeros(t,1);

     jj=1;
     do until jj>t;

    et=y[jj,1]-x[jj,.]*beta;

        loglik= (-0.5*ln(2*pi*sig2) - 0.5*et^2/sig2);

    lik_mat[jj,1]=loglik;

     jj=jj+1;
     endo;

     val=-1*sumc(lik_mat);

retp(val);
endp;


PROC TRANS(c0);

local  c1;

     c1=c0;
     c1[6,1]= exp(c0[6,1]);

retp(c1);
endp;
