@===================================================================@
@   REGRESSION MODEL    WITH AUTOCORRELATED DISTURBANCES            @
@          Model:   Y_t = X*beta + e_t,                             @
@          e_t = phi *e_{t-1} + v_t                                 @
@             v_t -- N(0,sigma^2)                                   @
@ MAXIMUM LIKELIHOOD ESTIMATION OF beta and  sigma (K+2 parameters) @
@===================================================================@
/* Chang-Jin Kim */

new;
library optmum;

     output file=mle_ato.out reset; output off;


     format /m1 /rd 9,5;
     load data[150,2]=gbs_ato.dta;

     Y=data[.,1];
     t=rows(y);
     X=ONES(t,1)~data[.,2];

     b_ini=inv(x'x)*x'y;
     e_hat=y-x*b_ini;
     s_ini=e_hat'e_hat/(t-1);

@----------INITIAL VALUES OF THE PARAMETERS@

prmtr_in=b_ini|0|s_ini;

@----------MAXIMUM LIKELIHOOD ESTIMATION@

{xout,fout,gout,cout}=optmum(&like_v,prmtr_in);

PRM_FNL=TRANS(xout);             @ Estimated  coefficients, constrained@
output on;

"==FINAL OUTPUT========================================================";

"code is--------";;cout;
"likelihood value is ";; -1*fout;
"Estimated parameters are:";
prm_fnl'; " ";
xout';
output off;


"Calculating Hessian..... Please be patient!!!!";
                  hout0=hessp(&like_v,xout);
                  hout=inv(hout0);

grdn_fnl=gradfd(&TRANS,xout);
Hsn_fnl=grdn_fnl*hout*grdn_fnl';
SD_fnl =sqrt(diag(Hsn_fnl)); @Standard errors of the estimated coefficients@

output on;
"Standard errors of parameters are:"; sd_fnl';
"===============================================================";
output off;




end;

@============END OF THE MAIN PROGRAM=================================@

proc like_v(prmtr1);

local beta, sig2, et, lik_mat,jj,val,loglik,prmtr,phi,vt,etl;

     prmtr=trans(prmtr1);

     beta=prmtr[1:2,1];
     phi=prmtr[3,1];

     sig2=prmtr[4,1];

     lik_mat=zeros(t,1);

     jj=2;
     do until jj>t;

     et=y[jj,1]-x[jj,.]*beta;
     etl=y[jj-1]-x[jj-1,.]*beta;

     vt= et-phi*etl;

        loglik= (-0.5*ln(2*pi*sig2) - 0.5*vt^2/sig2);

    lik_mat[jj,1]=loglik;

     jj=jj+1;
     endo;

     val=-1*sumc(lik_mat);

retp(val);
endp;


PROC TRANS(c0);

local  c1;

     c1=c0;
     c1[3,1]=c0[3,1]/(1+abs(c0[3,1]));
     c1[4,1]= exp(c0[4,1]);

retp(c1);
endp;
